@@ -43,7 +43,7 @@ Smb4KCustomOptionsDialog::Smb4KCustomOptionsDialog(const OptionsPtr &options, QW
     //
     // Set the title
     //
-    setWindowTitle(i18n("Custom Options"));
+    setWindowTitle(i18n("Custom Settings"));
 
     //
     // Set up the layout
@@ -58,7 +58,7 @@ Smb4KCustomOptionsDialog::Smb4KCustomOptionsDialog(const OptionsPtr &options, QW
     headerLayout->setContentsMargins(0, 0, 0, 0);
 
     QLabel *pixmap = new QLabel(header);
-    QPixmap preferencesPixmap = KDE::icon(QStringLiteral("preferences-system-network")).pixmap(KIconLoader::SizeHuge);
+    QPixmap preferencesPixmap = KDE::icon(QStringLiteral("settings-configure")).pixmap(KIconLoader::SizeHuge);
     pixmap->setPixmap(preferencesPixmap);
     pixmap->setAlignment(Qt::AlignCenter);
 
@@ -66,11 +66,11 @@ Smb4KCustomOptionsDialog::Smb4KCustomOptionsDialog(const OptionsPtr &options, QW
 
     switch (m_options->type()) {
     case Host: {
-        description = new QLabel(i18n("<p>Define custom options for host <b>%1</b> and all the shares it provides.</p>", m_options->displayString()), header);
+        description = new QLabel(i18n("<p>Define custom settings for host <b>%1</b> and all the shares it provides.</p>", m_options->displayString()), header);
         break;
     }
     case Share: {
-        description = new QLabel(i18n("<p>Define custom options for share <b>%1</b>.</p>", m_options->displayString()), header);
+        description = new QLabel(i18n("<p>Define custom settings for share <b>%1</b>.</p>", m_options->displayString()), header);
         break;
     }
     default: {
