@@ -33,8 +33,6 @@
 #include <QDebug>
 #include <QDir>
 #include <QFileInfo>
-#include <QTextCodec>
-#include <QTextStream>
 #include <QTimer>
 #include <QUdpSocket>
 
@@ -48,9 +46,6 @@
 #include <KCoreAddons/KShell>
 #include <KCoreAddons/KUser>
 #include <KI18n/KLocalizedString>
-#include <KIOCore/KDiskFreeSpaceInfo>
-#include <KIOCore/KIO/Global>
-#include <KIOCore/KIO/StatJob>
 #include <KIOCore/KMountPoint>
 #include <KWidgetsAddons/KMessageBox>
 
@@ -203,6 +198,11 @@ void Smb4KMounter::import(bool checkInaccessible)
         return;
     }
 
+    //
+    // Boolean to determine at the end if the mounted shares list changed
+    //
+    bool changed = false;
+
     //
     // Get the mountpoints that are present on the system
     //
@@ -306,8 +306,10 @@ void Smb4KMounter::import(bool checkInaccessible)
             //
             // Remove the share from the global list and notify the program
             //
-            removeMountedShare(share);
-            Q_EMIT unmounted(unmountedShare);
+            if (removeMountedShare(share)) {
+                changed = true;
+                Q_EMIT unmounted(unmountedShare);
+            }
 
             //
             // Report the unmounted share to the user if it is a single one
@@ -331,11 +333,6 @@ void Smb4KMounter::import(bool checkInaccessible)
         // Reset the number of newly unmounted shares
         //
         d->newlyUnmounted = 0;
-
-        //
-        // Tell the program the list of mounted shares changed
-        //
-        Q_EMIT mountedSharesListChanged();
     } else {
         //
         // Reset the number of newly unmounted shares
@@ -344,39 +341,103 @@ void Smb4KMounter::import(bool checkInaccessible)
     }
 
     //
-    // Now stat the imported shares to get information about them.
-    // Do not use Smb4KShare::canonicalPath() here, otherwise we might
-    // get lock-ups with inaccessible shares.
+    // Now process the imported shares
     //
     if (Smb4KHardwareInterface::self()->isOnline()) {
-        QMutableListIterator<SharePtr> it(d->importedShares);
-
-        while (it.hasNext()) {
-            SharePtr share = it.next();
-            SharePtr mountedShare = findShareByPath(share->path());
+        while (!d->importedShares.isEmpty()) {
+            SharePtr importedShare = d->importedShares.takeFirst();
+            SharePtr mountedShare = findShareByPath(importedShare->path());
 
             if (mountedShare) {
                 if (mountedShare->isInaccessible() && !checkInaccessible) {
-                    it.remove();
+                    importedShare.clear();
                     continue;
                 }
             }
 
-            QUrl url = QUrl::fromLocalFile(share->path());
-            KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
-            job->setDetails(KIO::StatNoDetails);
-            connect(job, SIGNAL(result(KJob *)), this, SLOT(slotStatResult(KJob *)));
+            //
+            // Check the imported share and retrieve some information about it
+            //
+            check(importedShare);
 
-            // Do not use addSubJob(), because that would confuse isRunning(), etc.
+            //
+            // Find out whether this is a share mounted by the user
+            //
+            if (importedShare->path().startsWith(Smb4KMountSettings::mountPrefix().path())
+                || (!importedShare->isInaccessible()
+                    && importedShare->canonicalPath().startsWith(QDir(Smb4KMountSettings::mountPrefix().path()).canonicalPath()))) {
+                importedShare->setForeign(false);
+            } else if (importedShare->path().startsWith(QDir::homePath())
+                       || (!importedShare->isInaccessible() && importedShare->canonicalPath().startsWith(QDir::home().canonicalPath()))) {
+                importedShare->setForeign(false);
+            } else if (importedShare->user().userId() == KUser(KUser::UseRealUserID).userId()
+                       && importedShare->group().groupId() == KUserGroup(KUser::UseRealUserID).groupId()) {
+                importedShare->setForeign(false);
+            } else {
+                importedShare->setForeign(true);
+            }
 
-            job->start();
+            //
+            // Search for a previously added mounted share and try to update it. If this fails,
+            // add the share to the global list.
+            //
+            if (!importedShare->isForeign() || Smb4KMountSettings::detectAllShares()) {
+                if (updateMountedShare(importedShare)) {
+                    SharePtr updatedShare = findShareByPath(importedShare->path());
+
+                    if (updatedShare) {
+                        Q_EMIT updated(updatedShare);
+                    }
+
+                    importedShare.clear();
+                } else {
+                    if (addMountedShare(importedShare)) {
+                        changed = true;
+
+                        // Remove the share from the list of shares that are to be remounted
+                        QMutableListIterator<SharePtr> s(d->remounts);
+
+                        while (s.hasNext()) {
+                            SharePtr remount = s.next();
+
+                            if (!importedShare->isForeign()
+                                && QString::compare(remount->url().toString(QUrl::RemoveUserInfo | QUrl::RemovePort),
+                                                    importedShare->url().toString(QUrl::RemoveUserInfo | QUrl::RemovePort),
+                                                    Qt::CaseInsensitive)
+                                    == 0) {
+                                Smb4KCustomOptionsManager::self()->removeRemount(remount);
+                                s.remove();
+                                break;
+                            } else {
+                                continue;
+                            }
+                        }
+
+                        Q_EMIT mounted(importedShare);
+
+                        // Do not notify the user on first import
+                        if (d->firstImportDone) {
+                            d->newlyMounted += 1;
+
+                            if (!isRunning() && d->importedShares.isEmpty()) {
+                                if (d->newlyMounted > 1) {
+                                    Smb4KNotification::sharesMounted(d->newlyMounted);
+                                } else {
+                                    Smb4KNotification::shareMounted(importedShare);
+                                }
+                                d->newlyMounted = 0;
+                            }
+                        }
+                    } else {
+                        d->newlyMounted = 0;
+                        importedShare.clear();
+                    }
+                }
+            } else {
+                importedShare.clear();
+            }
         }
 
-        //
-        // Set d->firstImportDone here only for the case that no mounted shares
-        // could be found. In all other cases d->firstImportDone will be set in
-        // slotStatResult().
-        //
         if (!d->firstImportDone && d->importedShares.isEmpty()) {
             d->firstImportDone = true;
         }
@@ -390,6 +451,13 @@ void Smb4KMounter::import(bool checkInaccessible)
             share.clear();
         }
     }
+
+    //
+    // Tell the program the list of mounted shares changed
+    //
+    if (changed) {
+        Q_EMIT mountedSharesListChanged();
+    }
 }
 
 void Smb4KMounter::mountShare(const SharePtr &share)
@@ -437,7 +505,7 @@ void Smb4KMounter::mountShare(const SharePtr &share)
         // Wake-On-LAN: Wake up the host before mounting
         //
         if (Smb4KSettings::enableWakeOnLAN()) {
-            OptionsPtr options = Smb4KCustomOptionsManager::self()->findOptions(KIO::upUrl(share->url()));
+            OptionsPtr options = Smb4KCustomOptionsManager::self()->findOptions(share->url().resolved(QStringLiteral("..")));
 
             if (options && options->wolSendBeforeMount()) {
                 Q_EMIT aboutToStart(WakeUp);
@@ -512,7 +580,8 @@ void Smb4KMounter::mountShare(const SharePtr &share)
             parentDirectory = Smb4KMountSettings::mountPrefix();
         } else {
             QUrl u = Smb4KMountSettings::mountPrefix();
-            parentDirectory = KIO::upUrl(u);
+            parentDirectory = u.resolved(QStringLiteral(".."));
+            ;
         }
 
         QFile f(parentDirectory.path());
@@ -529,7 +598,7 @@ void Smb4KMounter::mountShare(const SharePtr &share)
 
             while (!parentDirectory.matches(u, QUrl::StripTrailingSlash)) {
                 QFile(u.path()).setPermissions(permissions);
-                u = KIO::upUrl(u);
+                u = u.resolved(QStringLiteral(".."));
             }
         }
 
@@ -684,8 +753,8 @@ void Smb4KMounter::unmountShare(const SharePtr &share, bool silent)
             } else {
                 if (!silent) {
                     if (KMessageBox::warningYesNo(QApplication::activeWindow(),
-                                                  i18n("<qt><p>The share <b>%1</b> is mounted to <br><b>%2</b> and owned by user <b>%3</b>.</p>"
-                                                       "<p>Do you really want to unmount it?</p></qt>",
+                                                  i18n("<p>The share <b>%1</b> is mounted to <br><b>%2</b> and owned by user <b>%3</b>.</p>"
+                                                       "<p>Do you really want to unmount it?</p>",
                                                        share->displayString(),
                                                        share->path(),
                                                        share->user().loginName()),
@@ -927,11 +996,6 @@ void Smb4KMounter::timerEvent(QTimerEvent *)
         //
         // Check the size, accessibility, etc. of the shares
         //
-        // FIXME: Hopefully we can replace this with a recursive QFileSystemWatcher
-        // approach in the future. However, using the existing QFileSystemWatcher
-        // and a QDirIterator to add all the subdirectories of a share to the watcher
-        // seems to be too resource consuming...
-        //
         if (d->checkTimeout >= 2500 && d->importedShares.isEmpty()) {
             for (const SharePtr &share : mountedSharesList()) {
                 check(share);
@@ -1225,7 +1289,7 @@ bool Smb4KMounter::fillMountActionArgs(const SharePtr &share, QVariantMap &map)
     // Locking
     //
     if (Smb4KMountSettings::noLocking()) {
-        argumentsList << QStringLiteral("nolock");
+        argumentsList << QStringLiteral("nobrl");
     }
 
     //
@@ -1346,24 +1410,26 @@ bool Smb4KMounter::fillMountActionArgs(const SharePtr &share, QVariantMap &map)
     //
     // Mount options provided by the user
     //
-    if (!Smb4KMountSettings::customCIFSOptions().isEmpty()) {
-        // SECURITY: Only pass those arguments to mount.cifs that do not pose
-        // a potential security risk and that have not already been defined.
-        //
-        // This is, among others, the proper fix to the security issue reported
-        // by Heiner Markert (aka CVE-2014-2581).
-        QStringList allowedArgs = allowedMountArguments();
-        QStringList list = Smb4KMountSettings::customCIFSOptions().split(QStringLiteral(","), Qt::SkipEmptyParts);
-        QMutableStringListIterator it(list);
+    if (Smb4KMountSettings::useCustomCifsOptions()) {
+        if (!Smb4KMountSettings::customCIFSOptions().isEmpty()) {
+            // SECURITY: Only pass those arguments to mount.cifs that do not pose
+            // a potential security risk and that have not already been defined.
+            //
+            // This is, among others, the proper fix to the security issue reported
+            // by Heiner Markert (aka CVE-2014-2581).
+            QStringList allowedArgs = allowedMountArguments();
+            QStringList list = Smb4KMountSettings::customCIFSOptions().split(QStringLiteral(","), Qt::SkipEmptyParts);
+            QMutableStringListIterator it(list);
 
-        while (it.hasNext()) {
-            QString arg = it.next().section(QStringLiteral("="), 0, 0);
+            while (it.hasNext()) {
+                QString arg = it.next().section(QStringLiteral("="), 0, 0);
 
-            if (!allowedArgs.contains(arg)) {
-                it.remove();
+                if (!allowedArgs.contains(arg)) {
+                    it.remove();
+                }
+
+                argumentsList += list;
             }
-
-            argumentsList += list;
         }
     }
 
@@ -1446,7 +1512,7 @@ bool Smb4KMounter::fillMountActionArgs(const SharePtr &share, QVariantMap &map)
     //
     WorkgroupPtr workgroup = findWorkgroup(share->workgroupName());
 
-    if ((workgroup && !workgroup->dnsDiscovered()) && !share->workgroupName().isEmpty()) {
+    if ((workgroup && !workgroup->dnsDiscovered()) || (!workgroup && !share->workgroupName().trimmed().isEmpty())) {
         argumentsList << QStringLiteral("-W");
         argumentsList << KShell::quoteArg(share->workgroupName());
     }
@@ -1690,17 +1756,15 @@ bool Smb4KMounter::fillUnmountActionArgs(const SharePtr &, bool, bool, QVariantM
 
 void Smb4KMounter::check(const SharePtr &share)
 {
-    // Get the info about the usage, etc.
-    KDiskFreeSpaceInfo spaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(share->path());
+    d->storageInfo.setPath(share->path());
 
-    if (spaceInfo.isValid()) {
+    if (d->storageInfo.isValid() && d->storageInfo.isReady()) {
         // Accessibility
         share->setInaccessible(false);
 
         // Size information
-        share->setFreeDiskSpace(spaceInfo.available());
-        share->setTotalDiskSpace(spaceInfo.size());
-        share->setUsedDiskSpace(spaceInfo.used());
+        share->setFreeDiskSpace(d->storageInfo.bytesAvailable()); // Bytes available to the user, might be less that bytesFree()
+        share->setTotalDiskSpace(d->storageInfo.bytesTotal());
 
         // Get the owner an group, if possible.
         QFileInfo fileInfo(share->path());
@@ -1709,12 +1773,7 @@ void Smb4KMounter::check(const SharePtr &share)
         if (fileInfo.exists()) {
             share->setUser(KUser(static_cast<K_UID>(fileInfo.ownerId())));
             share->setGroup(KUserGroup(static_cast<K_GID>(fileInfo.groupId())));
-            share->setInaccessible(!(fileInfo.isDir() && fileInfo.isExecutable()));
         } else {
-            share->setInaccessible(true);
-            share->setFreeDiskSpace(0);
-            share->setTotalDiskSpace(0);
-            share->setUsedDiskSpace(0);
             share->setUser(KUser(KUser::UseRealUserID));
             share->setGroup(KUserGroup(KUser::UseRealUserID));
         }
@@ -1722,7 +1781,6 @@ void Smb4KMounter::check(const SharePtr &share)
         share->setInaccessible(true);
         share->setFreeDiskSpace(0);
         share->setTotalDiskSpace(0);
-        share->setUsedDiskSpace(0);
         share->setUser(KUser(KUser::UseRealUserID));
         share->setGroup(KUserGroup(KUser::UseRealUserID));
     }
@@ -1846,149 +1904,6 @@ void Smb4KMounter::slotOnlineStateChanged(bool online)
     }
 }
 
-void Smb4KMounter::slotStatResult(KJob *job)
-{
-    Q_ASSERT(job);
-
-    //
-    // Stat job
-    //
-    KIO::StatJob *statJob = static_cast<KIO::StatJob *>(job);
-
-    //
-    // Get the mountpoint
-    //
-    QString mountpoint = statJob->url().toDisplayString(QUrl::PreferLocalFile);
-
-    //
-    // Find the imported share
-    //
-    SharePtr importedShare;
-
-    for (int i = 0; i < d->importedShares.size(); ++i) {
-        if (QString::compare(d->importedShares.at(i)->path(), mountpoint) == 0) {
-            importedShare = d->importedShares.takeAt(i);
-            break;
-        }
-    }
-
-    //
-    // If the share should have vanished in the meantime, return here.
-    //
-    if (!importedShare) {
-        return;
-    }
-
-    //
-    // Add the size, user and group information
-    //
-    if (statJob->error() == 0 /* no error */) {
-        check(importedShare);
-    } else {
-        importedShare->setInaccessible(true);
-        importedShare->setFreeDiskSpace(0);
-        importedShare->setTotalDiskSpace(0);
-        importedShare->setUsedDiskSpace(0);
-        importedShare->setUser(KUser(KUser::UseRealUserID));
-        importedShare->setGroup(KUserGroup(KUser::UseRealUserID));
-    }
-
-    //
-    // Decide whether this is a share mounted by the user or by someone else.
-    //
-    QString canonicalMountPrefix = QDir(Smb4KMountSettings::mountPrefix().path()).canonicalPath();
-    QString canonicalHomePath = QDir::home().canonicalPath();
-
-    if (importedShare->path().startsWith(Smb4KMountSettings::mountPrefix().path()) || importedShare->canonicalPath().startsWith(canonicalMountPrefix)) {
-        //
-        // The path is below the mount prefix
-        //
-        importedShare->setForeign(false);
-    } else if (importedShare->path().startsWith(QDir::homePath()) || importedShare->canonicalPath().startsWith(canonicalHomePath)) {
-        //
-        // The path is below the home directory
-        //
-        importedShare->setForeign(false);
-    } else if (importedShare->user().userId() == KUser(KUser::UseRealUserID).userId()
-               && importedShare->group().groupId() == KUserGroup(KUser::UseRealUserID).groupId()) {
-        //
-        // The IDs are the same
-        //
-        importedShare->setForeign(false);
-    } else {
-        //
-        // The path is elsewhere. This is most certainly a foreign share.
-        //
-        importedShare->setForeign(true);
-    }
-
-    //
-    // Search for a previously added mounted share and try to update it. If this fails,
-    // add the share to the global list.
-    //
-    if (!importedShare->isForeign() || Smb4KMountSettings::detectAllShares()) {
-        if (updateMountedShare(importedShare)) {
-            SharePtr updatedShare = findShareByPath(importedShare->path());
-
-            if (updatedShare) {
-                Q_EMIT updated(updatedShare);
-            }
-
-            importedShare.clear();
-        } else {
-            if (addMountedShare(importedShare)) {
-                // Remove the share from the list of shares that are to be remounted
-                QMutableListIterator<SharePtr> s(d->remounts);
-
-                while (s.hasNext()) {
-                    SharePtr remount = s.next();
-
-                    if (!importedShare->isForeign()
-                        && QString::compare(remount->url().toString(QUrl::RemoveUserInfo | QUrl::RemovePort),
-                                            importedShare->url().toString(QUrl::RemoveUserInfo | QUrl::RemovePort),
-                                            Qt::CaseInsensitive)
-                            == 0) {
-                        Smb4KCustomOptionsManager::self()->removeRemount(remount);
-                        s.remove();
-                        break;
-                    } else {
-                        continue;
-                    }
-                }
-
-                // Tell the program and the user that the share was mounted. Also, reset the
-                // counter of newly mounted shares, if necessary.
-                d->newlyMounted += 1;
-                Q_EMIT mounted(importedShare);
-
-                if (!isRunning() && d->firstImportDone && d->importedShares.isEmpty() && d->newlyMounted == 1) {
-                    Smb4KNotification::shareMounted(importedShare);
-                }
-
-                QTimer::singleShot(250, this, [&]() {
-                    if (!isRunning()) {
-                        if (d->firstImportDone && d->importedShares.isEmpty() && d->newlyMounted > 1) {
-                            Smb4KNotification::sharesMounted(d->newlyMounted);
-                        }
-
-                        d->newlyMounted = 0;
-                    }
-                });
-
-                Q_EMIT mountedSharesListChanged();
-            } else {
-                importedShare.clear();
-            }
-        }
-    } else {
-        importedShare.clear();
-    }
-
-    if (!d->firstImportDone && d->importedShares.isEmpty()) {
-        d->firstImportDone = true;
-    }
-}
-
 void Smb4KMounter::slotAboutToChangeProfile()
 {
     //
@@ -2021,9 +1936,9 @@ void Smb4KMounter::slotActiveProfileChanged(const QString &newProfile)
         unmountAllShares(true);
 
         // Reset some variables.
+        // Don't touch d->firstImportDone here, because that remains true
         d->remountTimeout = 0;
         d->remountAttempts = 0;
-        d->firstImportDone = false;
         d->activeProfile = newProfile;
 
         // Restart the timer
