@@ -133,36 +133,10 @@ void Smb4KSyncJob::slotStartSynchronization()
     //
     if (Smb4KSettings::archiveMode()) {
         command << QStringLiteral("--archive");
-    } else {
-        if (Smb4KSettings::recurseIntoDirectories()) {
-            command << QStringLiteral("--recursive");
-        }
+    }
 
-        if (Smb4KSettings::preserveSymlinks()) {
-            command << QStringLiteral("--links");
-        }
-
-        if (Smb4KSettings::preservePermissions()) {
-            command << QStringLiteral("--perms");
-        }
-
-        if (Smb4KSettings::preserveTimes()) {
-            command << QStringLiteral("--times");
-        }
-
-        if (Smb4KSettings::preserveGroup()) {
-            command << QStringLiteral("--group");
-        }
-
-        if (Smb4KSettings::preserveOwner()) {
-            command << QStringLiteral("--owner");
-        }
-
-        if (Smb4KSettings::preserveDevicesAndSpecials()) {
-            // Alias -D
-            command << QStringLiteral("--devices");
-            command << QStringLiteral("--specials");
-        }
+    if (Smb4KSettings::recurseIntoDirectories()) {
+        command << QStringLiteral("--recursive");
     }
 
     if (Smb4KSettings::relativePathNames()) {
@@ -216,6 +190,10 @@ void Smb4KSyncJob::slotStartSynchronization()
         command << QStringLiteral("--ignore-existing");
     }
 
+    if (Smb4KSettings::preserveSymlinks()) {
+        command << QStringLiteral("--links");
+    }
+
     if (Smb4KSettings::transformSymlinks()) {
         command << QStringLiteral("--copy-links");
     }
@@ -244,6 +222,44 @@ void Smb4KSyncJob::slotStartSynchronization()
         command << QStringLiteral("--keep-dirlinks");
     }
 
+    if (Smb4KSettings::preservePermissions()) {
+        command << QStringLiteral("--perms");
+    }
+
+    if (Smb4KSettings::preserveACLs()) {
+        command << QStringLiteral("--acls");
+    }
+
+    if (Smb4KSettings::preserveExtendedAttributes()) {
+        command << QStringLiteral("--xattrs");
+    }
+
+    if (Smb4KSettings::preserveAccessTimes()) {
+        command << QStringLiteral("--atimes");
+    }
+
+    if (Smb4KSettings::preserveCreateTimes()) {
+        command << QStringLiteral("--crtimes");
+    }
+
+    if (Smb4KSettings::preserveOwner()) {
+        command << QStringLiteral("--owner");
+    }
+
+    if (Smb4KSettings::preserveGroup()) {
+        command << QStringLiteral("--group");
+    }
+
+    if (Smb4KSettings::preserveDevicesAndSpecials()) {
+        // Alias -D
+        command << QStringLiteral("--devices");
+        command << QStringLiteral("--specials");
+    }
+
+    if (Smb4KSettings::preserveTimes()) {
+        command << QStringLiteral("--times");
+    }
+
     if (Smb4KSettings::omitDirectoryTimes()) {
         command << QStringLiteral("--omit-dir-times");
     }
@@ -264,11 +280,11 @@ void Smb4KSyncJob::slotStartSynchronization()
     }
 
     if (Smb4KSettings::self()->useMaximalTransferSize()) {
-        command << QStringLiteral("--max-size=") + QString::number(Smb4KSettings::self()->maximalTransferSize()) + QStringLiteral("K");
+        command << QStringLiteral("--max-size=") + QString::number(Smb4KSettings::self()->maximalTransferSize()) + QStringLiteral("kB");
     }
 
     if (Smb4KSettings::self()->useMinimalTransferSize()) {
-        command << QStringLiteral("--min-size=") + QString::number(Smb4KSettings::self()->minimalTransferSize()) + QStringLiteral("K");
+        command << QStringLiteral("--min-size=") + QString::number(Smb4KSettings::self()->minimalTransferSize()) + QStringLiteral("kB");
     }
 
     if (Smb4KSettings::keepPartial()) {
@@ -280,7 +296,7 @@ void Smb4KSyncJob::slotStartSynchronization()
     }
 
     if (Smb4KSettings::useBandwidthLimit()) {
-        command << QStringLiteral("--bwlimit=") + QString::number(Smb4KSettings::bandwidthLimit()) + QStringLiteral("K");
+        command << QStringLiteral("--bwlimit=") + QString::number(Smb4KSettings::bandwidthLimit()) + QStringLiteral("kB");
     }
 
     //
@@ -345,9 +361,11 @@ void Smb4KSyncJob::slotStartSynchronization()
         command << QStringLiteral("--include-from=") + Smb4KSettings::includeFrom().path();
     }
 
-    if (!Smb4KSettings::customFilteringRules().isEmpty()) {
-        qDebug() << "Do we need to spilt the filtering rules into a list?";
-        command << Smb4KSettings::customFilteringRules();
+    if (Smb4KSettings::useCustomFilteringRules()) {
+        if (!Smb4KSettings::customFilteringRules().isEmpty()) {
+            qDebug() << "Do we need to spilt the filtering rules into a list?";
+            command << Smb4KSettings::customFilteringRules();
+        }
     }
 
     if (Smb4KSettings::useFFilterRule()) {
