@@ -66,39 +66,10 @@ Smb4KBookmarkMenu::Smb4KBookmarkMenu(int type, QObject *parent)
     //
     // Add the toplevel "Mount All Bookmarks" action.
     //
-    // Show it if there are no (non-empty) categories defined and additionally
-    // enable it, if not all bookmarks are already mounted.
-    //
-    m_toplevelMount = new QAction(KDE::icon(QStringLiteral("media-mount")), i18n("Mount All Bookmarks"), menu());
-    m_toplevelMount->setVisible(false);
-    m_toplevelMount->setEnabled(false);
+    m_toplevelMount = new QAction(KDE::icon(QStringLiteral("media-mount")), i18n("Mount Bookmarks"), menu());
     addAction(m_toplevelMount);
     m_mountActions->addAction(m_toplevelMount);
 
-    QStringList allCategories = Smb4KBookmarkHandler::self()->categoryList();
-
-    if (allCategories.isEmpty() || (allCategories.size() == 1 && allCategories.first().isEmpty())) {
-        m_toplevelMount->setVisible(true);
-
-        QList<BookmarkPtr> bookmarks = Smb4KBookmarkHandler::self()->bookmarksList();
-        int mountedBookmarks = 0;
-
-        for (const BookmarkPtr &bookmark : qAsConst(bookmarks)) {
-            QList<SharePtr> mountedShares = findShareByUrl(bookmark->url());
-
-            if (!mountedShares.isEmpty()) {
-                for (const SharePtr &share : qAsConst(mountedShares)) {
-                    if (!share->isForeign()) {
-                        mountedBookmarks++;
-                        break;
-                    }
-                }
-            }
-        }
-
-        m_toplevelMount->setEnabled(mountedBookmarks != bookmarks.size());
-    }
-
     //
     // Add a separator
     //
@@ -237,7 +208,7 @@ void Smb4KBookmarkMenu::addBookmarkToMenu(const BookmarkPtr &bookmark)
             //
             // The "Mount All Bookmarks" action for this category
             //
-            QAction *bookmarkCategoryMount = new QAction(KDE::icon(QStringLiteral("media-mount")), i18n("Mount All Bookmarks"), bookmarkMenu->menu());
+            QAction *bookmarkCategoryMount = new QAction(KDE::icon(QStringLiteral("media-mount")), i18n("Mount Bookmarks"), bookmarkMenu->menu());
             QMap<QString, QVariant> categoryMountInfo;
             categoryMountInfo[QStringLiteral("type")] = QStringLiteral("category_mount");
             categoryMountInfo[QStringLiteral("category")] = bookmark->categoryName();
@@ -384,18 +355,37 @@ void Smb4KBookmarkMenu::removeBookmarkFromMenu(const BookmarkPtr &bookmark)
 
 void Smb4KBookmarkMenu::adjustMountActions()
 {
-    //
-    // Enable and show the mount action(s)
-    //
-    QStringList allCategories = Smb4KBookmarkHandler::self()->categoryList();
+    QList<BookmarkPtr> toplevelBookmarks = Smb4KBookmarkHandler::self()->bookmarksList(QStringLiteral(""));
 
-    if (allCategories.isEmpty() || (allCategories.size() == 1 && allCategories.first().isEmpty())) {
-        m_toplevelMount->setVisible(true);
-
-        QList<BookmarkPtr> allBookmarks = Smb4KBookmarkHandler::self()->bookmarksList();
+    if (!toplevelBookmarks.isEmpty()) {
         int mountedBookmarks = 0;
 
-        for (const BookmarkPtr &bookmark : qAsConst(allBookmarks)) {
+        for (const BookmarkPtr &bookmark : qAsConst(toplevelBookmarks)) {
+            QList<SharePtr> mountedShares = findShareByUrl(bookmark->url());
+
+            for (const SharePtr &share : qAsConst(mountedShares)) {
+                if (!share->isForeign()) {
+                    mountedBookmarks++;
+                    break;
+                }
+            }
+        }
+
+        m_toplevelMount->setVisible(true);
+        m_toplevelMount->setEnabled(mountedBookmarks != toplevelBookmarks.size());
+    } else {
+        m_toplevelMount->setVisible(false);
+        m_toplevelMount->setEnabled(false);
+    }
+
+    QList<QAction *> allMountActions = m_mountActions->actions();
+    QStringList allCategories = Smb4KBookmarkHandler::self()->categoryList();
+    int mountedBookmarks = 0;
+
+    for (const QString &category : qAsConst(allCategories)) {
+        QList<BookmarkPtr> bookmarks = Smb4KBookmarkHandler::self()->bookmarksList(category);
+
+        for (const BookmarkPtr &bookmark : bookmarks) {
             QList<SharePtr> mountedShares = findShareByUrl(bookmark->url());
 
             if (!mountedShares.isEmpty()) {
@@ -408,40 +398,15 @@ void Smb4KBookmarkMenu::adjustMountActions()
             }
         }
 
-        m_toplevelMount->setEnabled(mountedBookmarks != allBookmarks.size());
-    } else {
-        m_toplevelMount->setVisible(false);
-        m_toplevelMount->setEnabled(false);
-
-        QList<QAction *> allMountActions = m_mountActions->actions();
-        int mountedBookmarks = 0;
-
-        for (const QString &category : qAsConst(allCategories)) {
-            QList<BookmarkPtr> bookmarks = Smb4KBookmarkHandler::self()->bookmarksList(category);
-
-            for (const BookmarkPtr &bookmark : bookmarks) {
-                QList<SharePtr> mountedShares = findShareByUrl(bookmark->url());
-
-                if (!mountedShares.isEmpty()) {
-                    for (const SharePtr &share : qAsConst(mountedShares)) {
-                        if (!share->isForeign()) {
-                            mountedBookmarks++;
-                            break;
-                        }
-                    }
-                }
+        for (QAction *action : allMountActions) {
+            if (action->data().toMap().value(QStringLiteral("categrory")).toString() == category
+                && action->data().toMap().value(QStringLiteral("type")) == QStringLiteral("category_mount")) {
+                action->setEnabled(bookmarks.size() != mountedBookmarks);
+                break;
             }
-
-            for (QAction *action : allMountActions) {
-                if (action->data().toMap().value(QStringLiteral("categrory")).toString() == category
-                    && action->data().toMap().value(QStringLiteral("type")) == QStringLiteral("category_mount")) {
-                    action->setEnabled(bookmarks.size() != mountedBookmarks);
-                    break;
-                }
-            }
-
-            mountedBookmarks = 0;
         }
+
+        mountedBookmarks = 0;
     }
 }
 
